#!/usr/bin/env python3
# http://tech.franzone.blog/2012/11/24/listing-imap-mailboxes-with-python/
import imaplib
import sys
import traceback

# Your IMAP Settings
host = ''
port = 993
user = ''
password = ''

imap = None
login_quotes = [
    'LOGIN completed',
    'Logged in'
]


def parse_mailbox(data):
    ''' parse_mailbox() will extract the three pieces of information from
    the items returned from a call to IMAP4.list(). The initial data
    array looks like:
    [b'(\\HasChildren) "/" NameOfFolder', ...]
    At least on my server. If the folder name has spaces then it will
    be quote delimited.
    '''
    flags, b, c = data.partition(' ')
    separator, b, name = c.partition(' ')
    return (flags, separator.replace('"', ''), name.replace('"', ''))

try:
    # Create the IMAP Client
    # imap = imaplib.IMAP4(host, port)
    if port in [25, 143]:
        imap = imaplib.IMAP4(host, port)
    else:
        imap = imaplib.IMAP4_SSL(host, port)

    # Login to the IMAP server
    resp, data = imap.login(user, password)
    print('Response : ' + resp)
    print('Data : ' + repr(data))
    print()
    if resp == 'OK' and bytes.decode(data[0]) in login_quotes:
        # List all mailboxes
        resp, data = imap.list('""', '*')
        if resp == 'OK':
            for mbox in data:
                flags, separator, name = parse_mailbox(bytes.decode(mbox))
                fmt = '{0}    : [Flags = {1}; Separator = {2}'
                print(fmt.format(name, flags, separator))
except:
    print('Unexpected error : {0}'.format(sys.exc_info()[0]))
    traceback.print_exc()
finally:
    if imap is not None:
        imap.logout()
