#!/usr/bin/env python3
#
# author: Claudio Driussi
#
from xml.etree import ElementTree as ET
import sys
import os
import json
import io
import base64

class ParseFPA():

    def __init__(self, file_in):
        tree = ET.parse(file_in)
        self.root = tree.getroot()
        self.head = self.root.find('FatturaElettronicaHeader')
        self.body = self.root.find('FatturaElettronicaBody')
        self.ddt = {}
        self.ordini = {}
        self.dir_attach = os.path.dirname(file_in)

        self.data = {}
        if self.root.attrib['versione'] == 'FSM10':
            self.s_fornitore()
            self.s_cliente()
            self.documento()
            self.s_righe()
        else:
            self.fornitore()
            self.cliente()
            self.documento()
            self.riferimenti()
            self.righe()
            self.iva()
            self.pagamenti()
        self.allegati()


    def fornitore(self):
        node = {}

        node['RAGSOC'] = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/Anagrafica/Denominazione//////////', '')
        if not node['RAGSOC']:
            n1 = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/Anagrafica/Nome//////////', '')
            n2 = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/Anagrafica/Cognome/////////', '')
            node['RAGSOC'] = '%s %s' % (n2, n1)

        node['IVACEE'] = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/IdFiscaleIVA/IdPaese/////////', '')
        node['PIVA'] = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/IdFiscaleIVA/IdCodice/////////', '')
        node['CODFIS'] = self.get_key(self.head, 'CedentePrestatore/DatiAnagrafici/CodiceFiscale//////////', '')

        node['VIA'] = self.get_key(self.head, 'CedentePrestatore/Sede/Indirizzo//////////', '')
        n1 = self.get_key(self.head, 'CedentePrestatore/Sede/NumeroCivico//////////', '')
        if n1:
            node['VIA'] += ', ' + n1

        node['CAP'] = self.get_key(self.head, 'CedentePrestatore/Sede/CAP//////////', '')
        node['CITTA'] = self.get_key(self.head, 'CedentePrestatore/Sede/Comune//////////', '')
        node['PROV'] = self.get_key(self.head, 'CedentePrestatore/Sede/Provincia//////////', '')

        node['TELEFONO'] = self.get_key(self.head, 'DatiTrasmissione/ContattiTrasmittente/Telefono//////////', '')
        node['EMAIL'] = self.get_key(self.head, 'DatiTrasmissione/ContattiTrasmittente/Email//////////', '')

        self.data['FORNITORE'] = node

    def s_fornitore(self):
        node = {}

        node['RAGSOC'] = self.get_key(self.head, 'CedentePrestatore/Denominazione//////////', '')
        if not node['RAGSOC']:
            n1 = self.get_key(self.head, 'CedentePrestatore/Nome//////////', '')
            n2 = self.get_key(self.head, 'CedentePrestatore/Cognome/////////', '')
            node['RAGSOC'] = '%s %s' % (n2, n1)

        node['IVACEE'] = self.get_key(self.head, 'CedentePrestatore/IdFiscaleIVA/IdPaese/////////', '')
        node['PIVA'] = self.get_key(self.head, 'CedentePrestatore/IdFiscaleIVA/IdCodice/////////', '')
        node['CODFIS'] = self.get_key(self.head, 'CedentePrestatore/CodiceFiscale//////////', '')

        node['VIA'] = self.get_key(self.head, 'CedentePrestatore/Sede/Indirizzo//////////', '')
        n1 = self.get_key(self.head, 'CedentePrestatore/Sede/NumeroCivico//////////', '')
        if n1:
            node['VIA'] += ', ' + n1

        node['CAP'] = self.get_key(self.head, 'CedentePrestatore/Sede/CAP//////////', '')
        node['CITTA'] = self.get_key(self.head, 'CedentePrestatore/Sede/Comune//////////', '')
        node['PROV'] = self.get_key(self.head, 'CedentePrestatore/Sede/Provincia//////////', '')

        self.data['FORNITORE'] = node


    def cliente(self):
        node = {}

        node['RAGSOC'] = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/Anagrafica/Denominazione//////////', '')
        if not node['RAGSOC']:
            n1 = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/Anagrafica/Nome//////////', '')
            n2 = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/Anagrafica/Cognome/////////', '')
            node['RAGSOC'] = '%s %s' % (n2, n1)

        node['IVACEE'] = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/IdFiscaleIVA/IdPaese/////////', '')
        node['PIVA'] = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/IdFiscaleIVA/IdCodice/////////', '')
        node['CODFIS'] = self.get_key(self.head, 'CessionarioCommittente/DatiAnagrafici/CodiceFiscale//////////', '')

        node['VIA'] = self.get_key(self.head, 'CessionarioCommittente/Sede/Indirizzo//////////', '')
        n1 = self.get_key(self.head, 'CessionarioCommittente/Sede/NumeroCivico//////////', '')
        if n1:
            node['VIA'] += ', ' + n1

        node['CAP'] = self.get_key(self.head, 'CessionarioCommittente/Sede/CAP//////////', '')
        node['CITTA'] = self.get_key(self.head, 'CessionarioCommittente/Sede/Comune//////////', '')
        node['PROV'] = self.get_key(self.head, 'CessionarioCommittente/Sede/Provincia//////////', '')

        node['CODICE'] = self.get_key(self.head, 'DatiTrasmissione/CodiceDestinatario///////////', '')
        node['EMAIL'] = self.get_key(self.head, 'DatiTrasmissione/PECDestinatario///////////', '')

        self.data['CLIENTE'] = node

    def s_cliente(self):
        node = {}

        node['IVACEE'] = self.get_key(self.head, 'CessionarioCommittente/IdentificativiFiscali/IdFiscaleIVA/IdPaese/////////', '')
        node['PIVA'] = self.get_key(self.head, 'CessionarioCommittente/IdentificativiFiscali/IdFiscaleIVA/IdCodice/////////', '')
        node['CODFIS'] = self.get_key(self.head, 'CessionarioCommittente/IdentificativiFiscali/CodiceFiscale//////////', '')

        node['RAGSOC'] = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Denominazione//////////', '')
        if not node['RAGSOC']:
            n1 = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Nome//////////', '')
            n2 = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Cognome/////////', '')
            node['RAGSOC'] = '%s %s' % (n2, n1)

        node['VIA'] = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Sede/Indirizzo//////////', '')
        n1 = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Sede/NumeroCivico//////////', '')
        if n1:
            node['VIA'] += ', ' + n1

        node['CAP'] = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Sede/CAP//////////', '')
        node['CITTA'] = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Sede/Comune//////////', '')
        node['PROV'] = self.get_key(self.head, 'CessionarioCommittente/AltriDatiIdentificativi/Sede/Provincia//////////', '')

        self.data['CLIENTE'] = node

    def documento(self):
        node = {}

        node['T_DOC'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/TipoDocumento///////////', '')
        node['D_DOC'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/Data///////////', '')
        node['N_DOC'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/Numero///////////', '')
        node['TOTALE'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/ImportoTotaleDocumento///////////', 0.0)

        n1 = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/ScontoMaggiorazione/Tipo//////////', '')
        if n1:
            node['SCONTO'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/ScontoMaggiorazione/Percentuale//////////', 0.0)
            node['SC_IMPO'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/ScontoMaggiorazione/Importo//////////', 0.0)
            if n1 == 'MG':
                node['SCONTO'] = -node['SCONTO']
                node['SC_IMPO'] = -node['SC_IMPO']
        node['ARROT'] = self.get_key(self.body, 'DatiGenerali/DatiGeneraliDocumento/Arrotondamento//////////', 0.0)

        self.data['DOCUMENTO'] = node

    def riferimenti(self):
        riferimenti = self.body.find('DatiGenerali')
        for item in riferimenti:
            if item.tag == 'DatiDDT':
                # n_riga = self.get_key(item, 'RiferimentoNumeroLinea/////////', 0)
                data = self.get_key(item, 'DataDDT/////////', '')
                num = self.get_key(item, 'NumeroDDT/////////', '')
                for n_riga in item.iter('RiferimentoNumeroLinea'):
                    self.ddt[int(n_riga.text)] = (data,num)
            if item.tag == 'DatiOrdineAcquisto':
                # n_riga = self.get_key(item, 'RiferimentoNumeroLinea/////////', 0)
                data = self.get_key(item, 'DataDDT/////////', '')
                num = self.get_key(item, 'NumeroDDT/////////', '')
                for n_riga in item.iter('RiferimentoNumeroLinea'):
                    self.ordini[int(n_riga.text)] = (data,num)

    def righe(self):
        node = []
        righe = self.body.find('DatiBeniServizi')
        for item in righe:
            if item.tag == 'DettaglioLinee':
                riga = {}
                n_riga = self.get_key(item, 'NumeroLinea/////////', 0)
                if n_riga in self.ordini:
                    riga['RIF_ORD'] = self.ordini[n_riga]
                if n_riga in self.ddt:
                    riga['RIF_DDT'] = self.ddt[n_riga]
                t = self.get_key(item, 'CodiceArticolo/CodiceTipo////////', '')
                if t:
                    riga['CODART'] = self.get_key(item, 'CodiceArticolo/CodiceValore////////', '')
                riga['DESCRIZIONE'] = self.get_key(item, 'Descrizione/////////', '')
                riga['UMIS'] = self.get_key(item, 'UnitaMisura/////////', '')
                riga['QTA'] = self.get_key(item, 'Quantita/////////', 0.0)
                riga['PRUNIT'] = self.get_key(item, 'PrezzoUnitario/////////', 0.0)

                t = self.get_key(item, 'ScontoMaggiorazione/Tipo//////////', '')
                if t:
                    riga['SCONTO'] = self.get_key(item, 'ScontoMaggiorazione/Percentuale//////////', 0.0)
                    riga['SC_IMPO'] = self.get_key(item, 'ScontoMaggiorazione/Importo//////////', 0.0)
                    if t == 'MG':
                        riga['SCONTO'] = -riga['SCONTO']
                        riga['SC_IMPO'] = -riga['SC_IMPO']

                riga['TOTALE'] = self.get_key(item, 'PrezzoTotale/////////', 0.0)
                riga['ALIVA'] = self.get_key(item, 'AliquotaIVA/////////', 0.0)
                t = self.get_key(item, 'Natura////////', '')
                if t:
                    riga['CODIVA'] = t
                node.append(riga)

        self.data['RIGHE'] = node

    def s_righe(self):
        node = []

        for item in self.body:
            if not item.tag == 'DatiBeniServizi':
                continue
            riga = {}
            riga['DESCRIZIONE'] = self.get_key(item, 'Descrizione/////////', '')
            riga['QTA'] = 1
            riga['PRUNIT'] = self.get_key(item, 'Importo/////////', 0.0)
            riga['TOTALE'] = riga['PRUNIT']
            riga['ALIVA'] = self.get_key(item, 'DatiIVA/Aliquota/////////', 0.0)
            riga['IMPOSTA'] = self.get_key(item, 'DatiIVA/Imposta/////////', 0.0)
            t = self.get_key(item, 'Natura////////', '')
            if t:
                riga['CODIVA'] = t
            node.append(riga)

        self.data['RIGHE'] = node

    def iva(self):
        node = []
        righe = self.body.find('DatiBeniServizi')
        for item in righe:
            if item.tag == 'DatiRiepilogo':
                riga = {}
                riga['ALIVA'] = self.get_key(item, 'AliquotaIVA/////////', 0.0)
                t = self.get_key(item, 'Natura/////////', '')
                if t:
                    riga['NATURA'] = t
                riga['IMPO'] = self.get_key(item, 'ImponibileImporto/////////', 0.0)
                riga['IVA'] = self.get_key(item, 'Imposta/////////', 0.0)
                riga['SPLIT'] = self.get_key(item, 'EsigibilitaIVA/////////', '')
                t = self.get_key(item, 'RiferimentoNormativo/////////', '')
                if t:
                    riga['CODIVA'] = t
                node.append(riga)

        self.data['IVA'] = node

    def pagamenti(self):
        node = []
        righe = self.body.find('DatiPagamento')
        if righe is not None:
            for item in righe:
                if item.tag == 'DettaglioPagamento':
                    riga = {}
                    riga['CODPAG'] = self.get_key(item, 'ModalitaPagamento/////////', '')
                    riga['SCADENZA'] = self.get_key(item, 'DataScadenzaPagamento/////////', '')
                    riga['IMPO'] = self.get_key(item, 'ImportoPagamento/////////', 0.0)
                    t = self.get_key(item, 'IBAN/////////', '')
                    if t:
                        riga['IBAN'] = t
                    node.append(riga)

            self.data['SCADENZE'] = node

    def allegati(self):
        node = []
        righe = self.body.findall('Allegati')
        for item in righe:
            riga = {}
            riga['NOME'] = self.get_key(item, 'NomeAttachment/////////', '')
            riga['NOME'] = win_file_name(riga['NOME'])
            fname = os.path.join(self.dir_attach, riga['NOME'])
            data = base64.b64decode(self.get_key(item, 'Attachment/////////', ''))
            with open(fname,'wb') as f:
                f.write(data)
                f.close()
            node.append(riga)
        self.data['ALLEGATI'] = node

    def get_key(self, element, key, default=""):

        key = key.rstrip('/')
        val = element.find(key)
        if val is None:
            return default

        # conversione automatica di tipo in funzione del tipo del parametro default
        #if type(default) is float:
        #    return float(val.text)
        try:
            return type(default)(val.text)
        except:
            pass
        return val.text

    def save(self, file_out):
        with open(file_out, 'w') as outfile:
            json.dump(self.data, outfile, indent=4)


def win_file_name(old):
    new = ""
    for char in old:
        if char in '\\/:*?"<>| ':
            char = '_'
        new += char
    return new

if __name__ == '__main__':
    # file_in = 'parse_fpa.xml'
    # file_in = 'parse_fpa_sempl2.xml'
    # file_in = 'parse_fpa_noiva.xml'
    file_in = 'parse_fpa_sempl3.xml'
    file_out = 'parse_fpa.json'

    if len(sys.argv) > 1:
        file_in = sys.argv[1]

    if len(sys.argv) > 2:
        file_out = sys.argv[2]

    f = ParseFPA(file_in)
    f.save(file_out)
