#
# xpp>csGetPar({ {"TIPO","Tipo etichetta",'1',,'M',{'1 Tipo 1','1 Tipo 2','1 Tipo 3'}} })
#!/usr/bin/python
# -*- coding: windows-1250 -*-

##
## tracciato standard di stampa etichette con stampante Sound110 o simili.
##
## per abilitare la richiesta di parametri, ad esempio il tipo di etichetta 
## la prima riga deve contenere la stringa xpp> seguita dal comando da abilitare
##

import sys
import time
import os

class csImpText(object):
    """Legge un tracciato nel formato esportato da Zefiro e lo trasforma in una
    lista di dizionari. Il file contiene records con un campo per riga, ogni record
    inizia con una chiave, ad esempio:

    *** LABEL ***
    CODICE    :C:G4903XX16X0001
    DESCRIZ   :C:4903XX+CASSETTO LISCIO   138,0X  398,0X18
    CODCTA    :C:GT
    PRUNIT    :N:0.33

    """
    def __init__(self, key):
        """Si passa la chiave di separazione dei records"""
        self.key = key

    def load_file(self, file):
        """Legge un file da disco e lo trasforma in una lista si stringhe"""
        f = open(file,'r')
        l = f.readlines()
        f.close()
        l = [i.strip('\n') for i in l]
        self.load_list(l)

    def load_list(self, l):
        """Legge una lista si stringhe nel formato previsto e le trasforma in una
        lista di dizionari"""
        self.data = []
        rec = {}
        for i in l:
            if i.find(self.key) == 0:
                if rec:
                    self.data.append(rec)
                    rec = {}
            else:
                t = i.split(':')
                if len(t) > 2:
                    rec[t[0].strip()] = t[2]
                pass
        if rec:
            self.data.append(rec)


class Labels(object):
    """Stampa le etichette, legge un file nel formato di esportazione di Zefiro e
    produce un file con lo script da inviare alla stampate.
    """
    def __init__(self, tipo='1'):
        """Il parametro 'tipo' identifica il tracciato da utilizzare"""
        self.tipo = tipo

    def lbl_out(self, data):
        """Si puo' passare un nome di file o direttamente una lista di dizionari"""
        if type(data).__name__=='str':
            imp = csImpText('*** LABEL ***')
            imp.load_file(data)
            data = imp.data

        if self.tipo == '1':
            return self.lbl_1(data)

        if self.tipo == '2':
            return self.lbl_1(data)

        if self.tipo == '3':
            return self.lbl_1(data)

    def lbl_1(self, data):
        """Tracciato standard"""
        st = 'I8,A\n'
        st += 'ZN\n'
        st += 'q390\n'
        st += 'OD\n'
        st += 'JF\n'
        st += 'ZT\n'
        st += 'Q240,25\n'
        for rec in data:
            if rec.has_key('EANCODE'):
                st += 'B370,220,2,E30,3,6,67,N,"%s"\n' %  rec['EANCODE']
                st += 'A370,140,2,2,2,2,N,"%s"\n' %  rec['EANCODE']
            else:
                st += 'B370,220,2,1,2,4,67,N,"%s"\n' %  rec['ARTVAR']
                st += 'A370,140,2,2,2,2,N,"%s"\n' %  rec['ARTVAR']
                pass
            st += 'A370,100,2,2,1,2,N,"%s"\n' % rec['DESCRIZ']
            if rec.has_key('PROMO'):
                p = rec['PROMO']
                st += 'A370,40,2,4,1,1,N,"%s"\n' % rec['PRUNIT']
                l = len(rec['PRUNIT']) * 16
                st += 'LO%d,28,%d,2\n' % (370-l, l)
            else:
                p = rec['PRUNIT']
            st += 'A260,50,2,4,2,2,N,"%8s"\n' % (' %s' % p, ) # allinea a dx 
            st += 'P%s\n' % rec['QTA']
        self.labels = st

    def close(self):
        """Scrive lo script da stampare ed attiva il comando di stampa"""
        fname = 'C:\\CS\\LABELS.TXT'
        with open(fname, 'w') as f:
            f.write(self.labels)

        time.sleep(1)
        # os.system('ssdal /p "Sound110" send "%s"' % fname )

if __name__ == '__main__':
    # se attivo la gestione dei parametri, il primo parametro indica il tipo di etichetta
    # tipo = sys.argv[1].split("=")[1]
    tipo = '1'

    file_in = 'C:\\CS\\LABELS.TMP'
    #if len(sys.argv) > 1:
    #    file_in = sys.argv[1]

    lbl = Labels(tipo)
    lbl.lbl_out(file_in)
    lbl.close()
